"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = Observable;

var _isObject = _interopRequireDefault(require("lodash/isObject"));

var _shallowequal = _interopRequireDefault(require("shallowequal"));

var _Watch = _interopRequireWildcard(require("./Watch"));

var _Notify = _interopRequireDefault(require("./Notify"));

var arrayMethods = ["push", "pop", "shift", "unshift", "splice", "sort", "reverse", "concat"];

function Observable(data, computed, isRoot) {
  if (isRoot === void 0) {
    isRoot = true;
  }

  var protoListener = {}; // 递归监控所有属性
  //TODO:

  if (Array.isArray(data)) {
    data.forEach(function (value, i) {
      if ((0, _isObject.default)(value) || Array.isArray(value)) {
        data[i] = Observable(value, null, false);
      }
    });
  } else if ((0, _isObject.default)(data)) {
    Object.keys(data).forEach(function (key) {
      var value = data[key];

      if ((0, _isObject.default)(value) || Array.isArray(value)) {
        data[key] = Observable(value, null, false);
      }
    });
  }

  if (computed) {
    Object.keys(computed).forEach(function (key) {
      if (key in data) {
        console.warn("The computed property \"" + key + "\" is already defined in data.");
      }
    });
  }

  var handler = {
    get: function get(target, key, proxy) {
      if (isRoot && key === "$watch") {
        return function (path, watchFn) {
          var watcher = new _Watch.default(proxy, path, watchFn);
          return function () {
            watcher.clearDeps();
          };
        };
      }

      var value = target[key];
      var isComputed = computed && !(key in target) && key in computed && computed[key];

      if (_Watch.CurrentWatchDep.current && !isComputed) {
        var watcher = _Watch.CurrentWatchDep.current;
        protoListener[key] = protoListener[key] || new _Notify.default();
        watcher.addDep(protoListener[key]);
      }

      return isComputed ? computed[key].call(proxy) : value;
    },
    set: function set(target, key, value) {
      //TODO: 新增元素重新监控
      var oldValue = target[key];
      target[key] = value;

      if (!(0, _shallowequal.default)(oldValue, value) && protoListener[key]) {
        protoListener[key].pub();
      }

      return true;
    }
  };
  return new Proxy(data, handler);
}