"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var isObject_1 = require("lodash/isObject");
var shallowequal_1 = require("shallowequal");
var Watch_1 = require("./Watch");
var Notify_1 = require("./Notify");
var arrayMethods = ["push", "pop", "shift", "unshift", "splice", "sort", "reverse", "concat"];
function Observable(data) {
    // 递归监控所有属性
    if (Array.isArray(data)) {
        data.forEach(function (value, i) {
            if (isObject_1.default(value) || Array.isArray(value)) {
                data[i] = Observable(value);
            }
        });
    }
    else if (isObject_1.default(data)) {
        Object.keys(data).forEach(function (key) {
            var value = data[key];
            if (isObject_1.default(value) || Array.isArray(value)) {
                data[key] = Observable(value);
            }
        });
    }
    var notify = new Notify_1.default();
    var handler = {
        get: function (target, key) {
            if (key === "$watch") {
                return function (path, watchFn) {
                    var watcher = new Watch_1.default(target, path, watchFn);
                    notify.sub(function () {
                        watcher.check();
                    });
                };
            }
            if (Array.isArray(target) && arrayMethods.indexOf(key) !== -1) {
                return function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    //TODO: 新增元素重新监控
                    var ret = target[key].apply(target, args);
                    notify.pub();
                    return ret;
                };
            }
            return target[key];
        },
        set: function (target, key, value) {
            //TODO: 新增元素重新监控
            var oldValue = target[key];
            target[key] = value;
            if (!shallowequal_1.default(oldValue, value))
                notify.pub();
            return true;
        },
    };
    return new Proxy(data, handler);
}
exports.default = Observable;
